@@ -90,14 +90,14 @@ function(classifier=NULL, data=NULL, pre=function(x, y=NULL) { list(features=x)
             }
             combinedpredictions = rbind.fill(lapply(1:nrow(preds), function(j) {
                 ss = preds[j,,drop=F]
-                ord = order(ss, decreasing = TRUE)
+                ord = order(unlist(ss), decreasing = TRUE)
                 data.frame(ids[j,,drop=F], algorithm=factor(names(ss)[ord]), score=as.numeric(ss)[ord], iteration=i, row.names = NULL)
             }))
         } else {
             tmp = cbind(tmp.names = unlist(lapply(pairpredictions, rownames)), rbind.fill(pairpredictions))
             merged = ddply(tmp, "tmp.names", function(x) colSums(x[,-1], na.rm = TRUE))[-1]
             combinedpredictions = rbind.fill(lapply(1:nrow(merged), function(j) {
-                ord = order(merged[j,], decreasing = TRUE)
+                ord = order(unlist(merged[j,]), decreasing = TRUE)
                 data.frame(ids[j,,drop=F], algorithm=factor(names(merged)[ord]), score=as.numeric(merged[j,])[ord], iteration=i, row.names = NULL)
             }))
         }
@@ -184,14 +184,14 @@ function(classifier=NULL, data=NULL, pre=function(x, y=NULL) { list(features=x)
             }
             combinedpredictions =  rbind.fill(lapply(1:nrow(preds), function(j) {
                 ss = preds[j,,drop=F]
-                ord = order(ss, decreasing = TRUE)
+                ord = order(unlist(ss), decreasing = TRUE)
                 data.frame(ids[j,,drop=F], algorithm=factor(names(ss)[ord]), score=as.numeric(ss)[ord], iteration=i, row.names = NULL)
             }))
         } else {
             tmp = cbind(tmp.names = unlist(lapply(pairpredictions, rownames)), rbind.fill(pairpredictions))
             merged = ddply(tmp, "tmp.names", function(x) colSums(x[,-1], na.rm = TRUE))[-1]
             combinedpredictions = rbind.fill(lapply(1:nrow(merged), function(j) {
-                ord = order(merged[j,], decreasing = TRUE)
+                ord = order(unlist(merged[j,]), decreasing = TRUE)
                 data.frame(ids[j,,drop=F], algorithm=factor(names(merged)[ord]), score=as.numeric(merged[j,])[ord], iteration=i, row.names = NULL)
             }))
         }
