@@ -24,16 +24,29 @@ function(data=NULL, estimator=makeLearner("regr.lm"), epsilon=0.1, maxit=1000) {
             splits = split(1:nrow(data$data), data$data[[s]])
             haveind = splits$`TRUE`
             wantind = splits$`FALSE`
-            task = makeRegrTask(id="imputation", target="target", data=cbind(data.frame(target=data$data[haveind,p]), data$data[haveind,][data$features]))
-            model = train(estimator, task=task)
-            data$data[wantind,p] = predict(model, newdata=data$data[wantind,][data$features])$data$response
+            if(is.null(data$algorithmFeatures)) {
+                task = makeRegrTask(id="imputation", target="target", data=cbind(data.frame(target=data$data[haveind,p]), data$data[haveind,][data$features]))
+                model = train(estimator, task=task)
+                data$data[wantind,p] = predict(model, newdata=data$data[wantind,][data$features])$data$response
+            } else {
+                task = makeRegrTask(id="imputation", target="target", data=cbind(data.frame(target=data$data[haveind,p]), data$data[haveind,][c(data$features, data$algorithmFeatures)]))
+                model = train(estimator, task=task)
+                data$data[wantind,p] = predict(model, newdata=data$data[wantind,][c(data$features, data$algorithmFeatures)])$data$response
+            }
 
             diff = Inf
             it = 1
             while(diff > epsilon) {
-                task = makeRegrTask(id="imputation", target="target", data=cbind(data.frame(target=data$data[[p]]), data$data[data$features]))
-                model = train(estimator, task=task)
-                preds = predict(model, newdata=data$data[wantind,][data$features])$data$response
+                if(is.null(data$algorithmFeatures)) {
+                    task = makeRegrTask(id="imputation", target="target", data=cbind(data.frame(target=data$data[[p]]), data$data[data$features]))
+                    model = train(estimator, task=task)
+                    preds = predict(model, newdata=data$data[wantind,][data$features])$data$response
+                } else {
+                    task = makeRegrTask(id="imputation", target="target", data=cbind(data.frame(target=data$data[[p]]), data$data[c(data$features, data$algorithmFeatures)]))
+                    model = train(estimator, task=task)
+                    preds = predict(model, newdata=data$data[wantind,][c(data$features, data$algorithmFeatures)])$data$response
+                }
+
                 diff = max(abs(preds - data$data[wantind,p]))
                 data$data[wantind,p] = preds
                 it = it + 1
@@ -48,3 +61,4 @@ function(data=NULL, estimator=makeLearner("regr.lm"), epsilon=0.1, maxit=1000) {
 
     return(data)
 }
+
