@@ -3,7 +3,7 @@ function(data=NULL, model=NULL, addCosts=NULL) {
     if(is.null(data) || is.null(model)) {
         stop("Need both data and model to calculate misclassification penalties!")
     }
-
+    
     if(attr(model, "hasPredictions")) {
         predictions = model$predictions
     } else {
@@ -17,12 +17,27 @@ function(data=NULL, model=NULL, addCosts=NULL) {
             predictions = model(data)
         }
     }
-
+    
     optfun = if(data$minimize) { min } else { max }
-    perfs = data$data[data$performance]
+    if(is.null(data$algorithmFeatures)) {
+        perfs = data$data[data$performance]
+    } else {
+        d = data$data[c(data$ids, data$algos, data$performance)]
+        perfs = convertLongToWide(data=d, timevar=data$algos, idvar=data$ids, prefix=paste(data$performance,".",sep=""))
+        perfs = perfs[unique(data$data[[data$algos]])]
+    }
+    
     opts = apply(perfs, 1, optfun)
-    predictions$iid = match(do.call(paste, predictions[data$ids]), do.call(paste, data$data[data$ids]))
-    predictions$pid = match(predictions$algorithm, data$performance)
+    if(is.null(data$algorithmFeatures)) {
+        predictions$iid = match(do.call(paste, predictions[data$ids]), do.call(paste, data$data[data$ids]))  
+        predictions$pid = match(predictions$algorithm, data$performance)
+    } else {
+        d = data$data[c(data$ids, data$algos, data$performance)]
+        d = convertLongToWide(data=d, timevar=data$algos, idvar=data$ids, prefix=paste(data$performance,".",sep=""), remove.id=FALSE)
+        predictions$iid = match(do.call(paste, predictions[data$ids]), do.call(paste, d[data$ids]))
+        predictions$pid = match(predictions$algorithm, unique(data$data[[data$algos]]))
+    }
+    
     predictions$score = apply(predictions, 1, function(x) {
         pid = as.numeric(x[["pid"]])
         if(is.na(pid)) {
@@ -37,3 +52,4 @@ function(data=NULL, model=NULL, addCosts=NULL) {
 }
 class(misclassificationPenalties) = "llama.metric"
 attr(misclassificationPenalties, "minimize") = TRUE
+
