@@ -5,6 +5,14 @@ test_that("regression predicts", {
         expect_equal(ss$algorithm, factor(c("c", "b")))
         expect_equal(ss$score, c(0, 1))
     })
+    
+    # same test with algorithm features
+    res = regression(algotestregressor, d.algo)
+    expect_equal(unique(res$predictions$id), 11:20)
+    by(res$predictions, res$predictions$id, function(ss) {
+        expect_equal(ss$algorithm, factor(c("c", "b")))
+        expect_equal(ss$score, c(0, 1))
+    })
 })
 
 test_that("regression returns predictor", {
@@ -16,6 +24,16 @@ test_that("regression returns predictor", {
         expect_equal(ss$algorithm, factor(c("c", "b")))
         expect_equal(ss$score, c(0, 1))
     })
+    
+    # same test with algorithm features
+    res = regression(algotestregressor, d.algo)
+    fold.algo$id = id=rep.int(1:10, rep.int(2, 10))
+    preds = res$predictor(fold.algo)
+    expect_equal(unique(preds$id), 1:10)
+    by(preds, preds$id, function(ss) {
+        expect_equal(ss$algorithm, factor(c("c", "b")))
+        expect_equal(ss$score, c(0, 1))
+    })
 })
 
 test_that("regression returns predictor that works without IDs", {
@@ -27,6 +45,16 @@ test_that("regression returns predictor that works without IDs", {
         expect_equal(ss$algorithm, factor(c("c", "b")))
         expect_equal(ss$score, c(0, 1))
     })
+    
+    # same test with algorithm features
+    res = regression(algotestregressor, d.algo)
+    fold.algo$id = rep.int(1:10, rep.int(2, 10))
+    preds = res$predictor(fold.algo[c(d.algo$features, d.algo$algorithmFeatures, d.algo$algos)])
+    expect_equal(unique(preds$id), 1:10)
+    by(preds, preds$id, function(ss) {
+        expect_equal(ss$algorithm, factor(c("c", "b")))
+        expect_equal(ss$score, c(0, 1))
+    })
 })
 
 test_that("regression raises error without regressor", {
@@ -47,8 +75,8 @@ test_that("regression raises error without train/test split", {
 test_that("regression allows to combine by max", {
     fold = data.frame(a=rep.int(0, 10), best=rep.int("b", 10), foo=rep.int(2, 10), bar=rep.int(1, 10))
     d = list(data=rbind(cbind(fold, id=1:10), cbind(fold, id=11:20)),
-        train=list(1:nrow(fold)), test=list(1:nrow(fold) + nrow(fold)),
-        features=c("a"), performance=c("foo", "bar"), minimize=F, ids=c("id"))
+             train=list(1:nrow(fold)), test=list(1:nrow(fold) + nrow(fold)),
+             features=c("a"), performance=c("foo", "bar"), minimize=F, ids=c("id"))
     class(d) = "llama.data"
     attr(d, "hasSplits") = TRUE
     res = regression(testregressor, d)
@@ -57,7 +85,7 @@ test_that("regression allows to combine by max", {
         expect_equal(ss$algorithm, factor(c("foo", "bar")))
         expect_equal(ss$score, c(2, 1))
     })
-
+    
     fold$id = 1:10
     preds = res$predictor(fold)
     expect_equal(unique(preds$id), 1:10)
@@ -74,7 +102,7 @@ test_that("regression allows stacking", {
         expect_equal(ss$algorithm, factor(c("b", "c")))
         expect_equal(ss$score, c(1, 0))
     })
-
+    
     fold$id = 1:10
     preds = res$predictor(fold)
     expect_equal(unique(preds$id), 1:10)
@@ -98,7 +126,7 @@ test_that("regression works with NA predictions", {
         expect_equal(ss$algorithm, factor(NA))
         expect_equal(ss$score, Inf)
     })
-
+    
     res = regression(natestregressor, d, combine=natestclassifier)
     expect_equal(unique(res$predictions$id), 11:20)
     by(res$predictions, res$predictions$id, function(ss) {
@@ -124,3 +152,4 @@ test_that("regression works with single algorithm", {
         expect_equal(ss$score, c(1))
     })
 })
+    
\ No newline at end of file
